package me.sonic.dash;

/**
 * Created by Leonie on 7/2/13.
 */
import android.view.SurfaceHolder;
import android.annotation.SuppressLint;
import android.graphics.Canvas;
import android.util.Log;

public class MainThread extends Thread {

    private final static int 	MAX_FPS = 50;
    private final static int 	MAX_FRAME_SKIPS = 5;
    private final static int 	FRAME_PERIOD = 1000 / MAX_FPS;

    private static final String TAG = MainThread.class.getSimpleName();

    // flag to hold game state
    private boolean running;

    private SurfaceHolder surfaceHolder;
    private MainGamePanel gamePanel;

    public MainThread(SurfaceHolder surfaceHolder, MainGamePanel gamePanel) {
        super();
        this.surfaceHolder = surfaceHolder;
        this.gamePanel = gamePanel;
    }

    public void setRunning(boolean running) {
        this.running = running;
    }
    @Override
    public void run() {
        Canvas canvas = null;
        Log.d(TAG, "Starting game loop");

        long beginTime=0;		// time when cycle begun
        long timeDiff=0;		// the time for the cycle to execute
        int sleepTime=0;		// ms to sleep (<0 if we're behind)
        int framesSkipped=0;	// number of frames being skipped

        while (running) {
            try {
                canvas = this.surfaceHolder.lockCanvas();
                synchronized (surfaceHolder) {
                    beginTime = System.currentTimeMillis();
                    framesSkipped = 0;
                    //update
                    this.gamePanel.update();
                    // draw
                    if (canvas != null)
                        this.gamePanel.onDraw(canvas);
                    // calculate how long the cycle took
                    timeDiff = System.currentTimeMillis() - beginTime;
                    // calculate the sleep time
                    sleepTime = (int)(FRAME_PERIOD - timeDiff);

                    if (sleepTime > 0) {
                        try {
                            Thread.sleep(sleepTime);
                        } catch (InterruptedException e) {}

                    }

                    while (sleepTime < 0 && framesSkipped < MAX_FRAME_SKIPS) {
                        this.gamePanel.update();
                        sleepTime += FRAME_PERIOD;
                        framesSkipped++;
                    }

                }
            } finally {

                if (canvas != null)
                    surfaceHolder.unlockCanvasAndPost(canvas);
            }
        }
        Log.d(TAG, "Game loop executed ");
    }
}