package com.example.game;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import android.content.Context;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.PorterDuff.Mode;
import android.util.Log;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;

public class MainGamePanel extends SurfaceView implements SurfaceHolder.Callback {
	
	private MainThread thread;
	private Sonic sonic;
	GestureListener gestures;
	private int screenwidth;
	private int screenheight;
	final private int MAX_ENEMIES = 10;
	private long currentRunTime;
	private long lastRunTime;
	protected int lastEnemyMovement;
	private long gameStart;
	List<Enemy> enemies;
	
	public MainGamePanel(Context context) {
		super(context);
		
		// adding the callback (this) to the surface holder to intercept events
		getHolder().addCallback(this);
		
		currentRunTime = 0;
		lastRunTime = 0;
		gameStart = System.currentTimeMillis();
		sonic = new Sonic(50, 50, 8);
		sonic.setStandingBitmap(BitmapFactory.decodeResource(getResources(), R.drawable.sonicstanding));
		sonic.setRollingBitmap(BitmapFactory.decodeResource(getResources(), R.drawable.sonicrolling));
		sonic.setMovingBitmap(1, BitmapFactory.decodeResource(getResources(), R.drawable.sonicmov1));
		sonic.setMovingBitmap(2, BitmapFactory.decodeResource(getResources(), R.drawable.sonicmov2));
		sonic.setMovingBitmap(3, BitmapFactory.decodeResource(getResources(), R.drawable.sonicmov3));
		sonic.setMovingBitmap(4, BitmapFactory.decodeResource(getResources(), R.drawable.sonicmov4));
		sonic.setMovingBitmap(5, BitmapFactory.decodeResource(getResources(), R.drawable.sonicmov5));
		sonic.setMovingBitmap(6, BitmapFactory.decodeResource(getResources(), R.drawable.sonicmov6));
		sonic.setMovingBitmap(7, BitmapFactory.decodeResource(getResources(), R.drawable.sonicmov7));
		sonic.setMovingBitmap(8, BitmapFactory.decodeResource(getResources(), R.drawable.sonicmov8));
		
		enemies = new ArrayList<Enemy>();
		
		// Create the gameloop thread
		thread = new MainThread(getHolder(), this);
		
		// make the GamePanel focusable so it can handle events
		setFocusable(true);
		
		// Gestures
		gestures = new GestureListener();
		final GestureDetector JumpSwipeDetect = new GestureDetector(gestures);
		//mGestureDetector = new GestureListener();
		this.setOnTouchListener(new OnTouchListener() {
	        @Override
	        public boolean onTouch(View v, MotionEvent event) {
	            JumpSwipeDetect.onTouchEvent(event);
	            if (gestures.lastSwipeDir == -1){
	            	if (!sonic.isJumping() && !sonic.isFalling())
	            		sonic.triggerJumping = true;
	            }
	            gestures.lastSwipeDir = 0;
	            return true;
	        }
	    });
		
	}
	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
		screenwidth = width;
		screenheight = height;
		sonic.setGroundY(height-100);
		Log.w("Screen size changed: ", "Width: "+screenwidth+", Height: "+screenheight);
	}
	
	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		thread.setRunning(true);
		thread.start();
	}
	
	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		boolean retry = true;
		while (retry) {
			try {
				thread.join();
				retry = false;
			} catch(InterruptedException e) {
				
			}
		}
	}

	private void collisions(){
		for (int i = 0; i < enemies.size(); i++){
			if (sonic.getX() + sonic.getWidth()/2 >= enemies.get(i).x - enemies.get(i).getWidth()/2 
					&& 
				sonic.getY() + sonic.getHeight()/2 >= enemies.get(i).y - enemies.get(i).getHeight()/2
					&&
				sonic.getX() - sonic.getWidth()/2 <= enemies.get(i).x + enemies.get(i).getWidth()/2
					&&
				sonic.getY() - sonic.getHeight() <= enemies.get(i).y + enemies.get(i).getHeight()/2){
					/* ToDo 
					 * Deze triggerCollision is snel gemaakt voor het testen van de collisions om te kunnen zien dat het werkte, 
					 * Dus dit moet nog veranderd worden!
					 * 
					 * Wel eerst de 2 verschillende enemies er in zetten (instant dead & ring loss)
					 */
					sonic.triggerCollision();
					Enemy.setDead(true);
			}
		}
	}
	
	protected void update() {
		currentRunTime = System.currentTimeMillis();
		collisions();
		if (lastEnemyMovement < 0)
			lastEnemyMovement = 0;
		sonic.update();
		Random ran = new Random();
		/* ToDo
		 * Punten / Ringen
		 */
		if (System.currentTimeMillis() - gameStart >= 10000){
			if (enemies.size() <= MAX_ENEMIES && lastEnemyMovement > 50 && canSpawn()){
				if (Math.round(ran.nextInt(20)) == 1 && lastRunTime != 0){
					/* ToDo
					 * Meerdere bit maps / verschillende enemies
					 */
					enemies.add(new Enemy(
						BitmapFactory.decodeResource(getResources(), R.drawable.ball), 
						screenwidth,
						screenheight-100));
				}
			}
		}
		lastEnemyMovement+=(int)(currentRunTime-lastRunTime);
		if (lastEnemyMovement > 50){
			for (int i = 0; i < enemies.size(); i++){
				enemies.get(i).x -= 10;
				if (enemies.get(i).x < 0-screenwidth/2){
					enemies.remove(i);
					Log.w("Update", "Enemy moved, "+enemies.get(i).x);
				}
			}
		}
		Log.w("Update", "Enemy test, "+lastEnemyMovement);
		lastRunTime = currentRunTime;
	}
	
	private boolean canSpawn() {
		for (int i = 0; i < enemies.size(); i++){
			if (enemies.get(i).x > screenwidth-enemies.get(i).width*5){
				return false;
			}
		}
		return true;
	}
	@Override
	protected void onDraw(Canvas canvas) {
		
		canvas.drawColor(0, Mode.CLEAR);
		sonic.draw(canvas);
		for (int i = 0; i < enemies.size(); i++){
			enemies.get(i).draw(canvas);
			Log.w("Draw", "Enemy drawed");
		}
	}
}