package com.example.game;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.PorterDuff.Mode;
import android.util.Log;
import android.view.MotionEvent;

public class Sonic {

	public boolean triggerJumping;
	
	private Bitmap bitmap_Standing;
	private Bitmap[] bitmap_Moving;
	private Bitmap bitmap_Rolling;
	private Bitmap bitmap_Current;
	private int x;
	private int y;
	private long lastRunTime;
	private long currentRunTime;
	private int fallUpdateTimePassed;
	private long fallTotalTimePassed;
	private int animUpdateTimePassed;;
	private Boolean falling;
	private Boolean jumping;
	private long startJump;
	private int JumpUpdateTimePassed;
	private Boolean rolling;
	private int currentAnimation;
	private int ground;
    

	public Sonic (int x, int y, int movAnimCount){
		this.x = x;
		this.y = y;
		this.bitmap_Moving = new Bitmap[movAnimCount];
		this.bitmap_Current = null;
		this.lastRunTime = System.currentTimeMillis();
		this.jumping = false;
		this.rolling = false;
		this.fallUpdateTimePassed = 0;
		this.JumpUpdateTimePassed = 0;
		this.ground = y+100;
		Log.w("Ground", "Ground "+this.ground);
	}
	
	public void setGroundY(int y){
		this.ground = y;
	}
	
	public void setMovingBitmap (int which, Bitmap bitmap){
		this.bitmap_Moving[which-1] = bitmap;
	}

	public void setStandingBitmap (Bitmap bitmap){
		this.bitmap_Standing = bitmap;

		if (bitmap_Current == null)
			bitmap_Current = bitmap_Standing;
	}
	
	public void setRollingBitmap (Bitmap bitmap){
		this.bitmap_Rolling = bitmap;
	}
	
	public int getX() {
		return x;
	}
	
	public int getY() {
		return y;
	}
	
	public void setX(int x) {
		this.x = x;
	}
	
	public void setY(int y) {
		this.y = y;
	}
	
	public int getHeight(){
		return bitmap_Current.getHeight();
	}
	
	public int getWidth(){
		return bitmap_Current.getWidth();
	}
	
	public boolean isJumping() {
		return jumping;
	}
	
	public boolean isFalling() {
		return falling;
	}
	
	private void falls(){
		this.y += Math.round(fallTotalTimePassed/10)+5;
		bitmap_Current = bitmap_Rolling;
	}
	
	public void update(){
		/* ToDo
		 * swipe down (spindash)
		 * ring counter
		 */
		currentRunTime = System.currentTimeMillis();
		animUpdateTimePassed += currentRunTime - lastRunTime;
		if (animUpdateTimePassed > 60) {
			currentAnimation++;
			if (currentAnimation > bitmap_Moving.length-1){
				currentAnimation = 0;
			}
			animUpdateTimePassed = 0;
		}
		if (y < ground && !jumping){
			falling = true;
			if (fallUpdateTimePassed >= 10){
				falls();
				fallUpdateTimePassed -= 10;
			}else{
				fallUpdateTimePassed += (currentRunTime - lastRunTime);
				fallTotalTimePassed += (currentRunTime - lastRunTime);
			}
		} else if (falling) {
			y = ground;
			falling = false;
			fallUpdateTimePassed = 0;
			fallTotalTimePassed = 0;
		}
		if (triggerJumping) {
			if (!jumping)
				jumping = true;
			else
				triggerJumping = false;
		}
		if (jumping) {
			if (JumpUpdateTimePassed >= 10){
				JumpUpdateTimePassed -= 10;
				jump();
			} else
				JumpUpdateTimePassed += (currentRunTime - lastRunTime);
		}
		if (!jumping && !falling && bitmap_Current != bitmap_Standing)
			bitmap_Current = bitmap_Moving[currentAnimation];
		lastRunTime = currentRunTime;
	}
	
	public void draw(Canvas canvas) {
		if (bitmap_Current != null)
			canvas.drawBitmap(bitmap_Current, x - (bitmap_Current.getWidth() / 2), y - (bitmap_Current.getHeight() / 2), null);
	}
	
	private void jump() {
		if (startJump == 0){
			startJump = System.currentTimeMillis();
			bitmap_Current = bitmap_Rolling;
		}
		if (System.currentTimeMillis() - startJump < 400){
			y-=Math.round((395 - (System.currentTimeMillis() - startJump)) / 15 + 5);
		} else {
			startJump = 0;
			jumping = false;
			falling = true;
		}
			
	}

	public void triggerCollision() {
		bitmap_Current = bitmap_Standing;
		
	}
	
}
